/**
 * @file SimpleProgram.h
 *
 * @date Aug 27, 2014
 * @author Hubert Degaudenzi
 *
 *
 * @copyright 2012-2020 Euclid Science Ground Segment
 *
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 3.0 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 */

/**
 * @addtogroup ElementsKernel ElementsKernel
 * @{
 */

#ifndef ELEMENTSKERNEL_ELEMENTSKERNEL_SIMPLE_PROGRAM_H_
#define ELEMENTSKERNEL_ELEMENTSKERNEL_SIMPLE_PROGRAM_H_

#include "ElementsKernel/Export.h"   // for ELEMENTS_API
#include "ElementsKernel/Logging.h"  // for Logging
#include "ElementsKernel/Path.h"     // for Item

namespace Elements {
enum class ExitCode;
}

namespace Elements {

class ELEMENTS_API SimpleProgram {

public:
  using ExitCode = Elements::ExitCode;
  using Logging  = Elements::Logging;

  ExitCode          run(int argc, char** argv) noexcept;
  const Path::Item& getProgramPath() const;
  const Path::Item& getProgramName() const;

protected:
  SimpleProgram() = default;
  virtual ~SimpleProgram();

  virtual ExitCode main()          = 0;
  virtual void     defineOptions() = 0;

private:
  void setup(int argc, char** argv);

private:
  Path::Item m_program_name;
  Path::Item m_program_path;
};

/** @example ElementsExamples/src/program/AnotherSimpleProgram.cpp
 * This is an example of how to use the SimpleProgram class.
 */

}  // namespace Elements

/**
 * @def MAIN(ELEMENTS_PROGRAM)
 * Macro which must be used to create a main in classes
 * that derived from Elements::SimpleProgram, i.e., these derived classes
 * must end with the following line:
 * @code
 *    MAIN(ELEMENTS_PROGRAM)
 * @endcode.
 * @param ELEMENTS_PROGRAM name of the main program class, derived from
 * the class Elements::SimpleProgram class.
 */
#define MAIN(ELEMENTS_PROGRAM)                                                                                         \
  ELEMENTS_API int main(int argc, char* argv[]) {                                                                      \
    auto               program   = ELEMENTS_PROGRAM();                                                                 \
    Elements::ExitCode exit_code = program.run(argc, argv);                                                            \
    return static_cast<Elements::ExitCodeType>(exit_code);                                                             \
  }

#endif  // ELEMENTSKERNEL_ELEMENTSKERNEL_SIMPLE_PROGRAM_H_

/**@}*/
